@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2009 Bastian Kleineidam
+# Copyright (C) 2000-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,17 +31,18 @@ class ProxySupport (object):
         and http:// URLs.
         """
         self.proxy = proxy
+        self.proxytype = "http"
         self.proxyauth = None
         if not self.proxy:
             return
         proxyargs = {"proxy": self.proxy}
-        if self.proxy[:7].lower() != "http://":
+        if self.proxytype not in ('http', 'https'):
             # Note that invalid proxies might raise TypeError in urllib2,
             # so make sure to stop checking at this point, not later.
-            msg = _("Proxy value `%(proxy)s' must start with 'http://'.") \
+            msg = _("Proxy value `%(proxy)s' must start with 'http:' or 'https:'.") \
                  % proxyargs
             raise LinkCheckerError(msg)
-        self.proxy = urllib.splittype(self.proxy)[1]
+        self.proxytype, self.proxy = urllib.splittype(self.proxy)
         self.proxy = urllib.splithost(self.proxy)[0]
         self.proxyauth, self.proxy = urllib.splituser(self.proxy)
         if self.ignore_proxy_host():
@@ -60,12 +61,9 @@ class ProxySupport (object):
         log.debug(LOG_CHECK, "using proxy %r", self.proxy)
 
     def ignore_proxy_host (self):
-        """Check if self.host is in the no-proxy-for ignore list."""
+        """Check if self.host is in the $no_proxy ignore list."""
         if urllib.proxy_bypass(self.host):
             return True
-        for ro in self.aggregate.config["noproxyfor"]:
-            if ro.search(self.host):
-                return True
         no_proxy = os.environ.get("no_proxy")
         if no_proxy:
             entries = [parse_host_port(x) for x in no_proxy.split(",")]
