@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2009 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,19 +11,17 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
 Handle telnet: links.
 """
 
 import telnetlib
-import urlparse
 import urllib
-
-import linkcheck
-import urlbase
+from . import urlbase
+from .. import log, LOG_CHECK
 
 
 class TelnetUrl (urlbase.UrlBase):
@@ -32,6 +30,10 @@ class TelnetUrl (urlbase.UrlBase):
     """
 
     def build_url (self):
+        """
+        Call super.build_url(), set default telnet port and initialize
+        the login credentials.
+        """
         super(TelnetUrl, self).build_url()
         # default port
         if self.port is None:
@@ -43,14 +45,21 @@ class TelnetUrl (urlbase.UrlBase):
             self.user, self.password = self.get_user_password()
 
     def local_check (self):
+        """
+        Warn about empty host names. Else call super.local_check().
+        """
         if not self.host:
             self.set_result(_("Host is empty"), valid=False)
             return
         super(TelnetUrl, self).local_check()
 
     def check_connection (self):
+        """
+        Open a telnet connection and try to login. Expected login
+        label is "login: ", expected password label is "Password: ".
+        """
         self.url_connection = telnetlib.Telnet()
-        if self.consumer.config.get("debug"):
+        if log.is_debug(LOG_CHECK):
             self.url_connection.set_debuglevel(1)
         self.url_connection.open(self.host, self.port)
         if self.user:
@@ -63,4 +72,10 @@ class TelnetUrl (urlbase.UrlBase):
         self.url_connection.write("exit\n")
 
     def can_get_content (self):
+        """
+        Telnet URLs have no content.
+
+        @return: False
+        @rtype: bool
+        """
         return False
