@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2009 Bastian Kleineidam
+# Copyright (C) 2000-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -126,7 +126,7 @@ class LCConfigParser (ConfigParser.RawConfigParser, object):
             if val:
                 self.config["warningregex"] = re.compile(val)
         if self.has_option(section, "warnsizebytes"):
-            val = self.get(section,"warnsizebytes")
+            val = self.get(section, "warnsizebytes")
             self.config["warnsizebytes"] = int(val)
         if self.has_option(section, "nntpserver"):
             self.config["nntpserver"] = self.get(section, "nntpserver")
@@ -134,19 +134,22 @@ class LCConfigParser (ConfigParser.RawConfigParser, object):
 
     def read_check_options (self, section):
         """Read check* options."""
-        if self.has_option(section,"checkhtml"):
+        if self.has_option(section, "checkhtml"):
             self.config["checkhtml"] = self.getboolean(section, "checkhtml")
-        if self.has_option(section,"checkcss"):
+        if self.has_option(section, "checkcss"):
             self.config["checkcss"] = self.getboolean(section, "checkcss")
-        if self.has_option(section,"checkhtmlw3"):
-            val = self.getboolean(section, "checkhtmlw3")
-            self.config["checkhtmlw3"] = val
-        if self.has_option(section,"checkcssw3"):
+        if self.has_option(section, "checkhtmlw3"):
+            self.config["checkhtmlw3"] = \
+               self.getboolean(section, "checkhtmlw3")
+        if self.has_option(section, "checkcssw3"):
             self.config["checkcssw3"] = self.getboolean(section, "checkcssw3")
         if self.has_option(section, "scanvirus"):
             self.config["scanvirus"] = self.getboolean(section, "scanvirus")
         if self.has_option(section, "clamavconf"):
             self.config["clamavconf"] = self.getboolean(section, "clamavconf")
+        if self.has_option(section, "cookies"):
+            self.config["sendcookies"] = self.config["storecookies"] = \
+                self.getboolean(section, "cookies")
 
     def read_authentication_config (self):
         """Read configuration options in section "authentication"."""
@@ -154,34 +157,33 @@ class LCConfigParser (ConfigParser.RawConfigParser, object):
         if self.has_option(section, "entry"):
             for val in read_multiline(self.get(section, "entry")):
                 auth = val.split()
-                if len(auth) != 3:
+                if len(auth) == 3:
+                    self.config.add_auth(pattern=auth[0], user=auth[1],
+                                         password=auth[2])
+                elif len(auth) == 2:
+                    self.config.add_auth(pattern=auth[0], user=auth[1])
+                else:
                     raise LinkCheckerError(LOG_CHECK,
                        _("missing auth part in entry %(val)r") % \
                        {"val": val})
-                self.config["authentication"].insert(0,
-                    {'pattern': re.compile(auth[0]),
-                     'user': auth[1],
-                     'password': auth[2]})
-        # backward compatibility
-        i = 1
-        while 1:
-            key = "entry%d" % i
-            if not self.has_option(section, key):
-                break
-            val = self.get(section, key)
-            auth = val.split()
-            log.warn(LOG_CHECK,
-              _("the entry%(num)d syntax is deprecated; use " \
-                "the new multiline configuration syntax") % {"num": i})
-            if len(auth) != 3:
+        # read login URL and field names
+        if self.has_option(section, "loginurl"):
+            val = self.get(section, "loginurl").strip()
+            if not (val.lower().startswith("http:") or
+                    val.lower().startswith("https:")):
                 raise LinkCheckerError(LOG_CHECK,
-                   _("missing auth part in entry %(val)r") % \
-                   {"val": val})
-            self.config["authentication"].insert(0,
-                {'pattern': re.compile(auth[0]),
-                 'user': auth[1],
-                 'password': auth[2]})
-            i += 1
+                _("Invalid login URL `%s'. Only " \
+                  "HTTP and HTTPS URLs are supported.") % val)
+            self.config["loginurl"] = val
+            self.config["storecookies"] = self.config["sendcookies"] = True
+        for key in ("loginuserfield", "loginpasswordfield"):
+            if self.has_option(section, key):
+                self.config[key] = self.get(section, key)
+        # read login extra fields
+        if self.has_option(section, "loginextrafields"):
+            for val in read_multiline(self.get(section, "loginextrafields")):
+                name, value = val.split(":", 1)
+                self.config["loginextrafields"][name] = value
 
     def read_filtering_config (self):
         """
@@ -192,34 +194,6 @@ class LCConfigParser (ConfigParser.RawConfigParser, object):
             for line in read_multiline(self.get(section, "nofollow")):
                 pat = get_link_pat(line, strict=0)
                 self.config["externlinks"].append(pat)
-        # backward compatibility
-        i = 1
-        while 1:
-            key = "nofollow%d" % i
-            if not self.has_option(section, key):
-                break
-            val = self.get(section, key)
-            log.warn(LOG_CHECK,
-              _("the nofollow%(num)d syntax is deprecated; use " \
-                "the new multiline configuration syntax") % {"num": i})
-            pat = get_link_pat(val, strict=0)
-            self.config["externlinks"].append(pat)
-            i += 1
-        if self.has_option(section, "noproxyfor"):
-            for val in read_multiline(self.get(section, "noproxyfor")):
-                self.config["noproxyfor"].append(re.compile(val))
-        # backward compatibility
-        i = 1
-        while 1:
-            key = "noproxyfor%d" % i
-            if not self.has_option(section, key):
-                break
-            log.warn(LOG_CHECK,
-                  _("the noproxyfor%(num)d syntax is deprecated; use " \
-                    "the new multiline configuration syntax") % {"num": i})
-            val = self.get(section, key)
-            self.config["noproxyfor"].append(re.compile(val))
-            i += 1
         if self.has_option(section, "ignorewarnings"):
             self.config['ignorewarnings'] = [f.strip() for f in \
                  self.get(section, 'ignorewarnings').split(',')]
@@ -227,20 +201,6 @@ class LCConfigParser (ConfigParser.RawConfigParser, object):
             for line in read_multiline(self.get(section, "ignore")):
                 pat = get_link_pat(line, strict=1)
                 self.config["externlinks"].append(pat)
-        # backward compatibility
-        i = 1
-        while 1:
-            key = "ignore%d" % i
-            if not self.has_option(section, key):
-                break
-            # backwards compatibility: split and ignore second part
-            val = self.get(section, key).split()[0]
-            log.warn(LOG_CHECK,
-              _("the ignore%(num)d syntax is deprecated; use " \
-                "the new multiline configuration syntax") % {"num": i})
-            pat = get_link_pat(val, strict=1)
-            self.config["externlinks"].append(pat)
-            i += 1
         if self.has_option(section, "internlinks"):
             pat = get_link_pat(self.get(section, "internlinks"))
             self.config["internlinks"].append(pat)
