@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2006 Bastian Kleineidam
+# Copyright (C) 2000-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,18 +11,16 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
 A SQL logger.
 """
 
-import time
 import os
-
-import linkcheck.logger
-import linkcheck.configuration
+from . import Logger
+from .. import url as urlutil
 
 
 def sqlify (s):
@@ -48,7 +46,7 @@ def intify (s):
     return 0
 
 
-class SQLLogger (linkcheck.logger.Logger):
+class SQLLogger (Logger):
     """
     SQL output, should work with any SQL database (not tested).
     """
@@ -73,17 +71,9 @@ class SQLLogger (linkcheck.logger.Logger):
         """
         Write start of checking info as sql comment.
         """
-        linkcheck.logger.Logger.start_output(self)
-        self.starttime = time.time()
+        super(SQLLogger, self).start_output()
         if self.has_part("intro"):
-            self.comment(_("created by %(app)s at %(time)s") %
-                        {"app": linkcheck.configuration.AppName,
-                         "time": linkcheck.strformat.strtime(self.starttime)})
-            self.comment(_("Get the newest version at %s") %
-                         linkcheck.configuration.Url)
-            self.comment(_("Write comments and bugs to %s") %
-                         linkcheck.configuration.Email)
-            self.check_date()
+            self.write_intro()
             self.writeln()
             self.flush()
 
@@ -91,13 +81,10 @@ class SQLLogger (linkcheck.logger.Logger):
         """
         Store url check info into the database.
         """
-        log_warnings = [x[1] for x in url_data.warnings]
-        log_infos = [x[1] for x in url_data.info]
-        self.writeln(u"insert into %(table)s(urlname,recursionlevel,"
+        self.writeln(u"insert into %(table)s(urlname,"
               "parentname,baseref,valid,result,warning,info,url,line,col,"
               "name,checktime,dltime,dlsize,cached) values ("
               "%(base_url)s,"
-              "%(recursion_level)d,"
               "%(url_parent)s,"
               "%(base_ref)s,"
               "%(valid)d,"
@@ -114,15 +101,14 @@ class SQLLogger (linkcheck.logger.Logger):
               "%(cached)d"
               ")%(separator)s" %
               {'table': self.dbname,
-               'base_url': sqlify(url_data.base_url or u""),
-               'recursion_level': url_data.recursion_level,
-               'url_parent': sqlify((url_data.parent_url or u"")),
-               'base_ref': sqlify((url_data.base_ref or u"")),
+               'base_url': sqlify(url_data.base_url),
+               'url_parent': sqlify((url_data.parent_url)),
+               'base_ref': sqlify((url_data.base_ref)),
                'valid': intify(url_data.valid),
                'result': sqlify(url_data.result),
-               'warning': sqlify(os.linesep.join(log_warnings)),
-               'info': sqlify(os.linesep.join(log_infos)),
-               'url': sqlify(linkcheck.url.url_quote(url_data.url or u"")),
+               'warning': sqlify(os.linesep.join(url_data.warnings)),
+               'info': sqlify(os.linesep.join(url_data.info)),
+               'url': sqlify(urlutil.url_quote(url_data.url)),
                'line': url_data.line,
                'column': url_data.column,
                'name': sqlify(url_data.name),
@@ -139,9 +125,5 @@ class SQLLogger (linkcheck.logger.Logger):
         Write end of checking info as sql comment.
         """
         if self.has_part("outro"):
-            self.stoptime = time.time()
-            duration = self.stoptime - self.starttime
-            self.comment(_("Stopped checking at %(time)s (%(duration)s)") %
-                 {"time": linkcheck.strformat.strtime(self.stoptime),
-                  "duration": linkcheck.strformat.strduration_long(duration)})
+            self.write_outro()
         self.close_fileoutput()
