@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2009 Bastian Kleineidam
+# Copyright (C) 2000-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,10 +18,11 @@
 Handle for mailto: links.
 """
 
-import cgi
 import re
 import urllib
+import urlparse
 import smtplib
+import socket
 from email._parseaddr import AddressList
 import sys
 
@@ -35,7 +36,15 @@ from .const import WARN_MAIL_NO_MX_HOST, \
 
 def getaddresses (addr):
     """Return list of email addresses from given field value."""
-    return [mail for name, mail in AddressList(addr).addresslist if mail]
+    parsed = [mail for name, mail in AddressList(addr).addresslist if mail]
+    if parsed:
+        addresses = parsed
+    elif addr:
+        # we could not parse any mail addresses, so try with the raw string
+        addresses = [addr]
+    else:
+        addresses = []
+    return addresses
 
 
 def is_quoted (addr):
@@ -75,13 +84,13 @@ class MailtoUrl (urlbase.UrlBase):
                 if not self.valid:
                     break
         else:
-            self.set_result(_("No mail addresses found in `%(url)s'.") % \
-                {"url": self.url}, valid=False, overwrite=False)
+            self.add_warning(_("No mail addresses found in `%(url)s'.") % \
+                {"url": self.url})
 
     def parse_addresses (self):
         """Parse all mail addresses out of the URL target. Also parses
         optional CGI headers like "?to=foo@example.org".
-        Stores parsed paddresses in the self.addresses set.
+        Stores parsed addresses in the self.addresses set.
         """
         # cut off leading mailto: and unquote
         url = urllib.unquote(self.base_url[7:])
@@ -108,7 +117,7 @@ class MailtoUrl (urlbase.UrlBase):
         if i < (len(url) - 1):
             self.addresses.update(getaddresses(url[:i]))
             try:
-                headers = cgi.parse_qs(url[(i+1):], strict_parsing=True)
+                headers = urlparse.parse_qs(url[(i+1):], strict_parsing=True)
                 for key, vals in headers.items():
                     if key.lower() in EMAIL_CGI:
                         # Only the first header value is added
@@ -227,7 +236,6 @@ class MailtoUrl (urlbase.UrlBase):
              an answer, print the verified address as an info.
              If not, print a warning.
         """
-        assert self.addresses
         for mail in sorted(self.addresses):
             self.check_smtp_domain(mail)
             if not self.valid:
@@ -321,7 +329,7 @@ class MailtoUrl (urlbase.UrlBase):
                     self.add_info(_("Verified address %(mail)s: %(info)s.") % d)
                 # check for 25x status code which means that the address
                 # could not be verified, but is sent anyway
-                elif 250 < status  < 260:
+                elif 250 < status < 260:
                     self.add_info(_("Unverified but presumably valid"
                                     " address %(mail)s: %(info)s.") % d)
                 else:
@@ -352,8 +360,9 @@ class MailtoUrl (urlbase.UrlBase):
         self.url_connection = None
         try:
             connection.quit()
-        except smtplib.SMTPException:
+        except (smtplib.SMTPException, socket.error):
             # ignore close errors
+            # socket.error is raised for example on timeouts
             pass
 
     def set_cache_keys (self):
