@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2009 Bastian Kleineidam
+# Copyright (C) 2000-2011 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,15 +29,17 @@ import socket
 import select
 
 from . import absolute_url, StoringHandler, get_url_from
-from ..cache import geoip
 from .. import (log, LOG_CHECK, LOG_CACHE, httputil, httplib2 as httplib,
-    strformat, LinkCheckerError, url as urlutil, trace, clamav, containers)
+  strformat, LinkCheckerError, url as urlutil, trace, clamav, winutil, geoip,
+  fileutil)
 from ..HtmlParser import htmlsax
-from ..htmlutil import linkparse, titleparse
-from .const import (WARN_URL_EFFECTIVE_URL, WARN_URL_UNICODE_DOMAIN,
-    WARN_URL_UNNORMED, WARN_URL_ERROR_GETTING_CONTENT,
+from ..htmlutil import linkparse
+from ..network import iputil
+from .const import (WARN_URL_EFFECTIVE_URL,
+    WARN_URL_ERROR_GETTING_CONTENT, WARN_URL_OBFUSCATED_IP,
     WARN_URL_ANCHOR_NOT_FOUND, WARN_URL_WARNREGEX_FOUND,
-    WARN_URL_CONTENT_TOO_LARGE, ExcList, ExcSyntaxList, ExcNoCacheList)
+    WARN_URL_CONTENT_SIZE_TOO_LARGE, WARN_URL_CONTENT_SIZE_ZERO,
+    WARN_URL_CONTENT_SIZE_UNEQUAL, ExcList, ExcSyntaxList, ExcNoCacheList)
 
 # helper alias
 unicode_safe = strformat.unicode_safe
@@ -50,15 +52,18 @@ def urljoin (parent, url, scheme):
     """
     if url.startswith(scheme+":"):
         return url
+    # work around a Python 2.6/3.1 bug cutting off characters when the URL
+    # begins with semicolon
+    if url.startswith(';'):
+        url = "./%s" % url
     return urlparse.urljoin(parent, url)
 
 
-def url_norm (url):
-    """
-    Wrapper for url.url_norm() to convert UnicodeError in LinkCheckerError.
-    """
+def url_norm (url, encoding=None):
+    """Wrapper for url.url_norm() to convert UnicodeError in
+    LinkCheckerError."""
     try:
-        return urlutil.url_norm(url)
+        return urlutil.url_norm(url, encoding=encoding)
     except UnicodeError:
         msg = _("URL has unparsable domain name: %(name)s") % \
             {"name": sys.exc_info()[1]}
@@ -68,9 +73,24 @@ def url_norm (url):
 class UrlBase (object):
     """An URL with additional information like validity etc."""
 
+    # file types that can be parsed recursively
+    ContentMimetypes = {
+        "text/html": "html",
+        "application/xhtml+xml": "html",
+        # Include PHP file which helps when checking local .php files.
+        # It does not harm other URL schemes like HTTP since HTTP servers
+        # should not send this content type. They send text/html instead.
+        "application/x-httpd-php": "html",
+        "text/css": "css",
+        "application/x-shockwave-flash": "swf",
+        "application/msword": "word",
+        "text/plain+linkchecker": "text",
+        "text/plain+opera": "opera",
+    }
+
     def __init__ (self, base_url, recursion_level, aggregate,
-                  parent_url = None, base_ref = None,
-                  line = -1, column = -1, name = u""):
+                  parent_url=None, base_ref=None, line=-1, column=-1,
+                  name=u"", url_encoding=None):
         """
         Initialize check data, and store given variables.
 
@@ -82,15 +102,15 @@ class UrlBase (object):
         @param line: line number of url in parent content
         @param column: column number of url in parent content
         @param name: name of url or empty
+        @param url_encoding: encoding of URL or None
         """
         self.init(base_ref, base_url, parent_url, recursion_level,
-                  aggregate, line, column, name)
+                  aggregate, line, column, name, url_encoding)
         self.reset()
         self.check_syntax()
 
-
     def init (self, base_ref, base_url, parent_url, recursion_level,
-              aggregate, line, column, name):
+              aggregate, line, column, name, url_encoding):
         """
         Initialize internal data.
         """
@@ -103,6 +123,7 @@ class UrlBase (object):
         self.line = line
         self.column = column
         self.name = name
+        self.encoding = url_encoding
         if self.base_ref:
             assert not urlutil.url_needs_quoting(self.base_ref), \
                    "unquoted base reference URL %r" % self.base_ref
@@ -112,8 +133,6 @@ class UrlBase (object):
         url = absolute_url(base_url, base_ref, parent_url)
         # assume file link if no scheme is found
         self.scheme = url.split(":", 1)[0] or "file"
-        # warn if URL is redirected (for commandline client)
-        self.warn_redirect = False
 
     def reset (self):
         """
@@ -141,6 +160,8 @@ class UrlBase (object):
         self.warnings = []
         # list of infos
         self.info = []
+        # content size
+        self.size = -1
         # download time
         self.dltime = -1
         # download size
@@ -160,6 +181,10 @@ class UrlBase (object):
         self.caching = True
         # title is either the URL or parsed from content
         self.title = None
+        # flag if content should be checked or not
+        self.do_check_content = True
+        # MIME content type
+        self.content_type = None
 
     def set_result (self, msg, valid=True, overwrite=False):
         """
@@ -195,16 +220,22 @@ class UrlBase (object):
 
     def set_title_from_content (self):
         """Set title of page the URL refers to.from page content."""
-        if self.valid and self.is_html():
+        if self.valid:
             try:
-                handler = titleparse.TitleFinder(self.get_content())
+                handler = linkparse.TitleFinder()
             except tuple(ExcList):
                 return
             parser = htmlsax.parser(handler)
             handler.parser = parser
             # parse
-            parser.feed(self.get_content())
-            parser.flush()
+            try:
+                parser.feed(self.get_content())
+                parser.flush()
+            except linkparse.StopParse, msg:
+                log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
+            # break cyclic dependencies
+            handler.parser = None
+            parser.handler = None
             if handler.title:
                 self.title = handler.title
 
@@ -257,21 +288,26 @@ class UrlBase (object):
         """
         self.result = cache_data["result"]
         self.has_result = True
+        anchor_changed = (self.anchor != cache_data["anchor"])
         for tag, msg in cache_data["warnings"]:
             # do not copy anchor warnings, since the current anchor
             # might have changed
-            if tag != WARN_URL_ANCHOR_NOT_FOUND:
-                self.add_warning(msg, tag=tag)
+            if anchor_changed and tag == WARN_URL_ANCHOR_NOT_FOUND:
+                pass
+            self.add_warning(msg, tag=tag)
         for info in cache_data["info"]:
             self.add_info(info)
         self.valid = cache_data["valid"]
         self.dltime = cache_data["dltime"]
         self.dlsize = cache_data["dlsize"]
         self.anchors = cache_data["anchors"]
+        self.content_type = cache_data["content_type"]
         self.cached = True
-        # recheck anchor
-        if self.valid and self.anchor:
-            self.check_anchor()
+        if anchor_changed and self.valid and self.anchor:
+            # recheck anchor
+            if self.check_anchor():
+                # a new warning has been added - remove cached flag
+                self.cached = False
 
     def get_cache_data (self):
         """Return all data values that should be put in the cache."""
@@ -282,6 +318,8 @@ class UrlBase (object):
                 "dltime": self.dltime,
                 "dlsize": self.dlsize,
                 "anchors": self.anchors,
+                "anchor": self.anchor,
+                "content_type": self.get_content_type(),
                }
 
     def get_alias_cache_data (self):
@@ -338,17 +376,7 @@ class UrlBase (object):
         url information self.base_url, self.parent_url and self.base_ref.
         """
         # norm base url - can raise UnicodeError from url.idna_encode()
-        base_url, is_idn = url_norm(self.base_url)
-        if is_idn:
-            self.add_warning(_("""URL %(url)r has a unicode domain name which
-                          is not yet widely supported. You should use
-                          the URL %(idna_url)r instead.""") % \
-                          {"url": self.base_url, "idna_url": base_url},
-                          tag=WARN_URL_UNICODE_DOMAIN)
-        elif self.base_url != base_url:
-            self.add_warning(
-              _("Base URL is not properly normed. Normed URL is %(url)s.") %
-               {'url': base_url}, tag=WARN_URL_UNNORMED)
+        base_url, is_idn = url_norm(self.base_url, self.encoding)
         # make url absolute
         if self.base_ref:
             # use base reference as parent url
@@ -389,6 +417,21 @@ class UrlBase (object):
                 raise LinkCheckerError(_("URL has invalid port %(port)r") %
                     {"port": str(self.port)})
             self.port = int(self.port)
+        self.check_obfuscated_ip()
+
+    def check_obfuscated_ip (self):
+        """Warn if host of this URL is obfuscated IP address."""
+        # check if self.host can be an IP address
+        if self.scheme not in ("ftp", "http", "mailto", "news", "nntp", "telnet"):
+            return
+        # check for obfuscated IP address
+        if iputil.is_obfuscated_ip(self.host):
+             ips = iputil.resolve_host(self.host)
+             if ips:
+                self.add_warning(
+                   _("URL %(url)s has obfuscated IP address %(ip)s") % \
+                   {"url": self.base_url, "ip": ips.pop()},
+                          tag=WARN_URL_OBFUSCATED_IP)
 
     def check (self):
         """Main check function for checking this URL."""
@@ -409,14 +452,18 @@ class UrlBase (object):
             self.close_connection()
 
     def add_country_info (self):
-        """
-        Try to ask GeoIP database for country info.
-        """
-        country = geoip.get_country(self.host)
-        if country is not None:
-            self.add_info(_("URL is located in %(country)s.") %
+        """Try to ask GeoIP database for country info."""
+        if self.host:
+            country = geoip.get_country(self.host)
+            if country is not None:
+                self.add_info(_("URL is located in %(country)s.") %
                 {"country": _(country)})
 
+    def add_size_info (self):
+        """Store size of URL content from meta info into self.size.
+        Must be implemented in subclasses."""
+        pass
+
     def local_check (self):
         """Local check function can be overridden in subclasses."""
         log.debug(LOG_CHECK, "Checking %s", self)
@@ -431,8 +478,8 @@ class UrlBase (object):
         log.debug(LOG_CHECK, "checking connection")
         try:
             self.check_connection()
+            self.add_size_info()
             self.add_country_info()
-            self.check_content()
         except tuple(ExcList):
             value = self.handle_exception()
             # make nicer error msg for unknown hosts
@@ -443,16 +490,18 @@ class UrlBase (object):
                 value = _('Bad HTTP response %(line)r') % {"line": str(value)}
             self.set_result(unicode_safe(value), valid=False)
         self.checktime = time.time() - check_start
-        # check recursion
-        try:
-            if self.allows_recursion():
-                self.parse_url()
-            # check content size
-            self.check_size()
-        except tuple(ExcList):
-            value = self.handle_exception()
-            self.add_warning(_("could not get content: %(msg)r") %
-                 {"msg": str(value)}, tag=WARN_URL_ERROR_GETTING_CONTENT)
+        if self.do_check_content:
+            # check content and recursion
+            try:
+                self.check_content()
+                if self.allows_recursion():
+                    self.parse_url()
+                # check content size
+                self.check_size()
+            except tuple(ExcList):
+                value = self.handle_exception()
+                self.add_warning(_("could not get content: %(msg)r") %
+                     {"msg": str(value)}, tag=WARN_URL_ERROR_GETTING_CONTENT)
 
     def close_connection (self):
         """
@@ -510,7 +559,7 @@ class UrlBase (object):
             log.debug(LOG_CHECK, "... no, cannot get content.")
             return False
         rec_level = self.aggregate.config["recursionlevel"]
-        if  rec_level >= 0 and self.recursion_level >= rec_level:
+        if rec_level >= 0 and self.recursion_level >= rec_level:
             log.debug(LOG_CHECK, "... no, maximum recursion level reached.")
             return False
         if self.extern[0]:
@@ -524,7 +573,7 @@ class UrlBase (object):
 
     def content_allows_robots (self):
         """
-        Return True if the content of this URL forbids robots to
+        Return False if the content of this URL forbids robots to
         search for recursive links.
         """
         if not self.is_html():
@@ -536,8 +585,11 @@ class UrlBase (object):
         parser = htmlsax.parser(handler)
         handler.parser = parser
         # parse
-        parser.feed(self.get_content())
-        parser.flush()
+        try:
+            parser.feed(self.get_content())
+            parser.flush()
+        except linkparse.StopParse, msg:
+            log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
         # break cyclic dependencies
         handler.parser = None
         parser.handler = None
@@ -546,25 +598,32 @@ class UrlBase (object):
     def get_anchors (self):
         """Store list of anchors for this URL. Precondition: this URL is
         an HTML resource."""
-        handler = linkparse.LinkFinder(self.get_content(),
+        log.debug(LOG_CHECK, "Getting HTML anchors %s", self)
+        handler = linkparse.LinkFinder(self.add_anchor,
                                    tags={'a': [u'name'], None: [u'id']})
         parser = htmlsax.parser(handler)
         handler.parser = parser
         # parse
-        parser.feed(self.get_content())
-        parser.flush()
+        try:
+            parser.feed(self.get_content())
+            parser.flush()
+        except linkparse.StopParse, msg:
+            log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
         # break cyclic dependencies
         handler.parser = None
         parser.handler = None
-        self.anchors = handler.urls[:]
+
+    def add_anchor (self, url, line, column, name, base):
+        """Add anchor URL."""
+        self.anchors.append((url, line, column, name, base))
 
     def check_anchor (self):
         """If URL was valid and has an anchor, check it. A warning is
-        logged if the anchor is not found.
+        logged and True is returned if the anchor is not found.
         """
         if not self.aggregate.config["anchors"]:
             return
-        log.debug(LOG_CHECK, "checking anchor %r", self.anchor)
+        log.debug(LOG_CHECK, "checking anchor %r in %s", self.anchor, self.anchors)
         if any(x for x in self.anchors if x[0] == self.anchor):
             return
         anchors = u",".join(u"`%s'" % x[0] for x in self.anchors)
@@ -572,6 +631,7 @@ class UrlBase (object):
         msg = u"%s %s" % (_("Anchor `%(name)s' not found.") % args,
                           _("Available anchors: %(anchors)s.") % args)
         self.add_warning(msg, tag=WARN_URL_ANCHOR_NOT_FOUND)
+        return True
 
     def set_extern (self, url):
         """
@@ -599,6 +659,13 @@ class UrlBase (object):
         self.extern = (1, 0)
         return
 
+    def get_content_type (self):
+        """Return content MIME type or empty string.
+        Should be overridden in subclasses."""
+        if self.content_type is None:
+            self.content_type = u""
+        return self.content_type
+
     def can_get_content (self):
         """Indicate wether url get_content() can be called."""
         return True
@@ -608,14 +675,15 @@ class UrlBase (object):
         if self.data is None:
             log.debug(LOG_CHECK, "Get content of %r", self.url)
             t = time.time()
-            self.data = self.read_content()
+            self.data, self.dlsize = self.read_content()
             self.dltime = time.time() - t
-            self.dlsize = len(self.data)
         return self.data
 
     def read_content (self):
-        """Return data for this URL. Can be overridden in subclasses."""
-        return self.url_connection.read()
+        """Return data and data size for this URL.
+        Can be overridden in subclasses."""
+        data = self.url_connection.read()
+        return data, len(data)
 
     def check_content (self):
         """Check content data for warnings, syntax errors, viruses etc."""
@@ -623,20 +691,11 @@ class UrlBase (object):
             return
         if self.is_html():
             self.set_title_from_content()
-            self.get_anchors()
+            if self.aggregate.config["anchors"]:
+                self.get_anchors()
         if self.anchor:
             self.check_anchor()
-        warningregex = self.aggregate.config["warningregex"]
-        if warningregex:
-            log.debug(LOG_CHECK, "checking content")
-            try:
-                match = warningregex.search(self.get_content())
-                if match:
-                    self.add_warning(_("Found %(match)r in link contents.") %
-                       {"match": match.group()}, tag=WARN_URL_WARNREGEX_FOUND)
-            except tuple(ExcList):
-                value = self.handle_exception()
-                self.set_result(unicode_safe(value), valid=False)
+        self.check_warningregex()
         # is it an intern URL?
         if not self.extern[0]:
             # check HTML/CSS syntax
@@ -652,18 +711,40 @@ class UrlBase (object):
             if self.aggregate.config["scanvirus"]:
                 self.scan_virus()
 
+    def check_warningregex (self):
+        warningregex = self.aggregate.config["warningregex"]
+        if warningregex:
+            log.debug(LOG_CHECK, "checking content")
+            try:
+                match = warningregex.search(self.get_content())
+                if match:
+                    self.add_warning(_("Found %(match)r in link contents.") %
+                       {"match": match.group()}, tag=WARN_URL_WARNREGEX_FOUND)
+            except tuple(ExcList):
+                value = self.handle_exception()
+                self.set_result(unicode_safe(value), valid=False)
+
     def check_size (self):
+        """Check content size if it is zero or larger than a given
+        maximum size.
         """
-        If a maximum size was given, call this function to check it
-        against the content size of this url.
-        """
-        maxbytes = self.aggregate.config["warnsizebytes"]
-        if maxbytes is not None and self.dlsize >= maxbytes:
-            self.add_warning(
+        if self.dlsize == 0:
+            self.add_warning(_("Content size is zero."),
+                             tag=WARN_URL_CONTENT_SIZE_ZERO)
+        else:
+            maxbytes = self.aggregate.config["warnsizebytes"]
+            if maxbytes is not None and self.dlsize >= maxbytes:
+                self.add_warning(
                    _("Content size %(dlsize)s is larger than %(maxbytes)s.") %
                         {"dlsize": strformat.strsize(self.dlsize),
                          "maxbytes": strformat.strsize(maxbytes)},
-                          tag=WARN_URL_CONTENT_TOO_LARGE)
+                          tag=WARN_URL_CONTENT_SIZE_TOO_LARGE)
+        if self.size != -1 and self.dlsize != -1 and self.dlsize != self.size:
+                self.add_warning(_("Download size (%(dlsize)d Byte) "
+                        "does not equal content size (%(size)d Byte).") %
+                        {"dlsize": self.dlsize,
+                         "size": self.size},
+                          tag=WARN_URL_CONTENT_SIZE_UNEQUAL)
 
     def check_html (self):
         """Check HTML syntax of this page (which is supposed to be HTML)
@@ -774,7 +855,7 @@ class UrlBase (object):
             h.putheader('Content-Length', str(len(body)))
             h.endheaders()
             h.send(body)
-            r = h.getresponse()
+            r = h.getresponse(True)
             if r.getheader('X-W3C-Validator-Status', 'Invalid') == 'Valid':
                 self.add_info(u"W3C Validator: %s" % _("valid CSS syntax"))
                 return
@@ -807,41 +888,37 @@ class UrlBase (object):
         self.parse_html()
 
     def get_user_password (self):
+        """Get tuple (user, password) from configured authentication.
+        Both user and password can be None.
         """
-        Get tuple (user, password) from configured authentication.
-        Both user and password can be None if not specified.
-        """
-        for auth in self.aggregate.config["authentication"]:
-            if auth['pattern'].match(self.url):
-                return auth['user'], auth['password']
-        return None, None
+        return self.aggregate.config.get_user_password(self.url)
 
     def parse_html (self):
-        """
-        Parse into HTML content and search for URLs to check.
+        """Parse into HTML content and search for URLs to check.
         Found URLs are added to the URL queue.
         """
         log.debug(LOG_CHECK, "Parsing HTML %s", self)
         # construct parser object
-        handler = linkparse.LinkFinder(self.get_content())
+        handler = linkparse.LinkFinder(self.add_url)
         parser = htmlsax.parser(handler)
         handler.parser = parser
         # parse
-        parser.feed(self.get_content())
-        parser.flush()
+        try:
+            parser.feed(self.get_content())
+            parser.flush()
+        except linkparse.StopParse, msg:
+            log.debug(LOG_CHECK, "Stopped parsing: %s", msg)
         # break cyclic dependencies
         handler.parser = None
         parser.handler = None
-        for url, line, column, name, codebase in handler.urls:
-            if codebase:
-                base_ref = codebase
-            else:
-                base_ref = handler.base_ref
-            base_ref = urlutil.url_norm(base_ref)[0]
-            url_data = get_url_from(url,
-                  self.recursion_level+1, self.aggregate, parent_url=self.url,
-                  base_ref=base_ref, line=line, column=column, name=name)
-            self.aggregate.urlqueue.put(url_data)
+
+    def add_url (self, url, line, column, name, base):
+        """Queue URL data for checking."""
+        base_ref = urlutil.url_norm(base)[0]
+        url_data = get_url_from(url, self.recursion_level+1, self.aggregate,
+            parent_url=self.url, base_ref=base_ref, line=line, column=column,
+            name=name)
+        self.aggregate.urlqueue.put(url_data)
 
     def parse_opera (self):
         """Parse an opera bookmark file."""
@@ -908,6 +985,38 @@ class UrlBase (object):
                          parent_url=self.url)
             self.aggregate.urlqueue.put(url_data)
 
+    def parse_word (self):
+        """Parse a word file for hyperlinks."""
+        if not winutil.has_word():
+            return
+        filename = self.get_temp_filename()
+        # open word file and parse hyperlinks
+        try:
+            app = winutil.get_word_app()
+            try:
+                doc = winutil.open_wordfile(app, filename)
+                try:
+                    for link in doc.Hyperlinks:
+                        url_data = get_url_from(link.Address,
+                                 self.recursion_level+1, self.aggregate,
+                                 parent_url=self.url, name=link.TextToDisplay)
+                        self.aggregate.urlqueue.put(url_data)
+                finally:
+                    winutil.close_wordfile(doc)
+            finally:
+                winutil.close_word_app(app)
+        except winutil.Error, msg:
+            log.warn(LOG_CHECK, "Error parsing word file: %s", msg)
+
+    def get_temp_filename (self):
+        """Get temporary filename for content to parse."""
+        # store content in temporary file
+        fd, filename = fileutil.get_tempfile(suffix='.doc', prefix='lc_')
+        try:
+            fd.write(self.get_content())
+        finally:
+            fd.close()
+
     def serialized (self):
         """
         Return serialized url check data as unicode string.
@@ -930,6 +1039,7 @@ class UrlBase (object):
             u"line=%d" % self.line,
             u"column=%d" % self.column,
             u"name=%r" % self.name,
+            u"anchor=%r" % self.anchor,
            ])
 
     def get_intern_pattern (self):
@@ -960,7 +1070,7 @@ class UrlBase (object):
         """
         return u"<%s >" % self.serialized()
 
-    def to_wire (self):
+    def to_wire_dict (self):
         """Return a simplified transport object for logging.
 
         The transport object must contain these attributes:
@@ -976,10 +1086,12 @@ class UrlBase (object):
           name of URL (eg. filename or link name)
         - url_data.parent_url: unicode or None
           Parent URL
-        - url_data.base_ref: unicode or None
+        - url_data.base_ref: unicode
           HTML base reference URL of parent
-        - url_data.url: unicode or None
+        - url_data.url: unicode
           Fully qualified URL.
+        - url_data.domain: unicode
+          URL domain part.
         - url_data.checktime: int
           Number of seconds needed to check this link, default: zero.
         - url_data.dltime: int
@@ -992,8 +1104,12 @@ class UrlBase (object):
           Line number of this URL at parent document, or -1
         - url_data.column: int
           Column number of this URL at parent document, or -1
+        - url_data.cache_url_key: unicode
+          Cache key for this URL.
+        - url_data.content_type: unicode
+          MIME content type for URL content.
         """
-        return containers.AttrDict(valid=self.valid,
+        return dict(valid=self.valid,
           extern=self.extern[0],
           cached=self.cached,
           result=self.result,
@@ -1004,6 +1120,7 @@ class UrlBase (object):
           base_ref=self.base_ref or u"",
           base_url=self.base_url or u"",
           url=self.url or u"",
+          domain=(self.urlparts[1] if self.urlparts else u""),
           checktime=self.checktime,
           dltime=self.dltime,
           dlsize=self.dlsize,
@@ -1011,10 +1128,46 @@ class UrlBase (object):
           line=self.line,
           column=self.column,
           cache_url_key=self.cache_url_key,
+          content_type=self.get_content_type(),
         )
 
+    def to_wire (self):
+        return CompactUrlData(self.to_wire_dict())
+
 
 def filter_tidy_errors (errors):
     """Filter certain errors from HTML tidy run."""
     return [x for x in errors if not \
         (x.severity=='W' and x.message=='<table> lacks "summary" attribute')]
+
+
+urlDataAttr = [
+    'valid',
+    'extern',
+    'cached',
+    'result',
+    'warnings',
+    'name',
+    'title',
+    'parent_url',
+    'base_ref',
+    'base_url',
+    'url',
+    'domain',
+    'checktime',
+    'dltime',
+    'dlsize',
+    'info',
+    'line',
+    'column',
+    'cache_url_key',
+    'content_type',
+]
+
+class CompactUrlData (object):
+    __slots__ = urlDataAttr
+
+    def __init__(self, wired_url_data):
+        '''Set all attributes according to the dictionnary wired_url_data'''
+        for attr in urlDataAttr:
+            setattr(self, attr, wired_url_data[attr])
