@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2006-2009 Bastian Kleineidam
+# Copyright (C) 2006-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 """
 URL checking functions.
 """
-import time
 from . import task
 from ..cache import urlqueue
 
@@ -62,7 +61,7 @@ class Checker (task.CheckedTask):
                     self.urlqueue.task_done(url_data)
                 self.setName(self.origname)
         except urlqueue.Empty:
-            time.sleep(0.1)
+            pass
 
     def check_url_data (self, url_data):
         """Check one URL data instance."""
