@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2003, 2004 Nominum, Inc.
+# Copyright (C) 2003-2007, 2009, 2010 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -30,7 +30,7 @@ def _exponent_of(what, desc):
             exp = i - 1
             break
     if exp is None or exp < 0:
-        raise linkcheck.dns.exception.DNSSyntaxError, "%s value out of bounds" % desc
+        raise linkcheck.dns.exception.DNSSyntaxError("%s value out of bounds" % desc)
     return exp
 
 def _float_to_tuple(what):
@@ -70,10 +70,10 @@ def _encode_size(what, desc):
 def _decode_size(what, desc):
     exponent = what & 0x0F
     if exponent > 9:
-        raise linkcheck.dns.exception.DNSSyntaxError, "bad %s exponent" % desc
+        raise linkcheck.dns.exception.DNSSyntaxError("bad %s exponent" % desc)
     base = (what & 0xF0) >> 4
     if base > 9:
-        raise linkcheck.dns.exception.DNSSyntaxError, "bad %s base" % desc
+        raise linkcheck.dns.exception.DNSSyntaxError("bad %s base" % desc)
     return long(base) * pow(10, exponent)
 
 class LOC(linkcheck.dns.rdata.Rdata):
@@ -165,16 +165,13 @@ class LOC(linkcheck.dns.rdata.Rdata):
             if '.' in t:
                 (seconds, milliseconds) = t.split('.')
                 if not seconds.isdigit():
-                    raise linkcheck.dns.exception.DNSSyntaxError, \
-                          'bad latitude seconds value'
+                    raise linkcheck.dns.exception.DNSSyntaxError('bad latitude seconds value')
                 latitude[2] = int(seconds)
                 if latitude[2] >= 60:
-                    raise linkcheck.dns.exception.DNSSyntaxError, \
-                          'latitude seconds >= 60'
+                    raise linkcheck.dns.exception.DNSSyntaxError('latitude seconds >= 60')
                 l = len(milliseconds)
                 if l == 0 or l > 3 or not milliseconds.isdigit():
-                    raise linkcheck.dns.exception.DNSSyntaxError, \
-                          'bad latitude milliseconds value'
+                    raise linkcheck.dns.exception.DNSSyntaxError('bad latitude milliseconds value')
                 if l == 1:
                     m = 100
                 elif l == 2:
@@ -189,7 +186,7 @@ class LOC(linkcheck.dns.rdata.Rdata):
         if t == 'S':
             latitude[0] *= -1
         elif t != 'N':
-            raise linkcheck.dns.exception.DNSSyntaxError, 'bad latitude hemisphere value'
+            raise linkcheck.dns.exception.DNSSyntaxError('bad latitude hemisphere value')
 
         longitude[0] = tok.get_int()
         t = tok.get_string()
@@ -199,16 +196,13 @@ class LOC(linkcheck.dns.rdata.Rdata):
             if '.' in t:
                 (seconds, milliseconds) = t.split('.')
                 if not seconds.isdigit():
-                    raise linkcheck.dns.exception.DNSSyntaxError, \
-                          'bad longitude seconds value'
+                    raise linkcheck.dns.exception.DNSSyntaxError('bad longitude seconds value')
                 longitude[2] = int(seconds)
                 if longitude[2] >= 60:
-                    raise linkcheck.dns.exception.DNSSyntaxError, \
-                          'longitude seconds >= 60'
+                    raise linkcheck.dns.exception.DNSSyntaxError('longitude seconds >= 60')
                 l = len(milliseconds)
                 if l == 0 or l > 3 or not milliseconds.isdigit():
-                    raise linkcheck.dns.exception.DNSSyntaxError, \
-                          'bad longitude milliseconds value'
+                    raise linkcheck.dns.exception.DNSSyntaxError('bad longitude milliseconds value')
                 if l == 1:
                     m = 100
                 elif l == 2:
@@ -223,33 +217,32 @@ class LOC(linkcheck.dns.rdata.Rdata):
         if t == 'W':
             longitude[0] *= -1
         elif t != 'E':
-            raise linkcheck.dns.exception.DNSSyntaxError, 'bad longitude hemisphere value'
+            raise linkcheck.dns.exception.DNSSyntaxError('bad longitude hemisphere value')
 
         t = tok.get_string()
         if t[-1] == 'm':
             t = t[0 : -1]
-        altitude = float(t) * 100.0     # m -> cm
+        altitude = float(t) * 100.0    # m -> cm
 
-        (ttype, value) = tok.get()
-        if ttype != linkcheck.dns.tokenizer.EOL and ttype != linkcheck.dns.tokenizer.EOF:
+        token = tok.get().unescape()
+        if not token.is_eol_or_eof():
+            value = token.value
             if value[-1] == 'm':
                 value = value[0 : -1]
-            size = float(value) * 100.0 # m -> cm
-            (ttype, value) = tok.get()
-            if ttype != linkcheck.dns.tokenizer.EOL and ttype != linkcheck.dns.tokenizer.EOF:
+            size = float(value) * 100.0      # m -> cm
+            token = tok.get().unescape()
+            if not token.is_eol_or_eof():
+                value = token.value
                 if value[-1] == 'm':
                     value = value[0 : -1]
-                hprec = float(value) * 100.0    # m -> cm
-                (ttype, value) = tok.get()
-                if ttype != linkcheck.dns.tokenizer.EOL and ttype != linkcheck.dns.tokenizer.EOF:
+                hprec = float(value) * 100.0   # m -> cm
+                token = tok.get().unescape()
+                if not token.is_eol_or_eof():
+                    value = token.value
                     if value[-1] == 'm':
                         value = value[0 : -1]
-                        vprec = float(value) * 100.0    # m -> cm
-                        (ttype, value) = tok.get()
-                        if ttype != linkcheck.dns.tokenizer.EOL and \
-                               ttype != linkcheck.dns.tokenizer.EOF:
-                            raise linkcheck.dns.exception.DNSSyntaxError, \
-                                  "expected EOL or EOF"
+                        vprec = float(value) * 100.0   # m -> cm
+                        tok.get_eol()
 
         return cls(rdclass, rdtype, latitude, longitude, altitude,
                    size, hprec, vprec)
@@ -295,13 +288,13 @@ class LOC(linkcheck.dns.rdata.Rdata):
         else:
             latitude = -1 * float(0x80000000L - latitude) / 3600000
         if latitude < -90.0 or latitude > 90.0:
-            raise linkcheck.dns.exception.FormError, "bad latitude"
+            raise linkcheck.dns.exception.FormError("bad latitude")
         if longitude > 0x80000000L:
             longitude = float(longitude - 0x80000000L) / 3600000
         else:
             longitude = -1 * float(0x80000000L - longitude) / 3600000
         if longitude < -180.0 or longitude > 180.0:
-            raise linkcheck.dns.exception.FormError, "bad longitude"
+            raise linkcheck.dns.exception.FormError("bad longitude")
         altitude = float(altitude) - 10000000.0
         size = _decode_size(size, "size")
         hprec = _decode_size(hprec, "horizontal precision")
