@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2009 Bastian Kleineidam
+# Copyright (C) 2000-2011 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -94,26 +94,26 @@ class TextLogger (Logger):
         Write generic start checking info.
         """
         super(TextLogger, self).start_output()
-        self.starttime = time.time()
         if self.has_part('intro'):
-            self.writeln(configuration.AppInfo)
-            self.writeln(configuration.Freeware)
-            self.writeln(_("Get the newest version at %(url)s") %
-                         {'url': configuration.Url})
-            self.writeln(_("Write comments and bugs to %(email)s") %
-                         {'email': configuration.Email})
-            self.check_date()
-            self.writeln()
-            self.writeln(_("Start checking at %s") %
-                         strformat.strtime(self.starttime))
-            self.flush()
+            self.write_intro()
+        self.flush()
+
+    def write_intro (self):
+        self.writeln(configuration.AppInfo)
+        self.writeln(configuration.Freeware)
+        self.writeln(_("Get the newest version at %(url)s") %
+                     {'url': configuration.Url})
+        self.writeln(_("Write comments and bugs to %(url)s") %
+                     {'url': configuration.SupportUrl})
+        self.check_date()
+        self.writeln()
+        self.writeln(_("Start checking at %s") %
+                     strformat.strtime(self.starttime))
 
     def log_url (self, url_data):
         """
         Write url checking info.
         """
-        if self.has_part('id'):
-            self.write_id()
         if self.has_part('url'):
             self.write_url(url_data)
         if url_data.name and self.has_part('name'):
@@ -142,7 +142,7 @@ class TextLogger (Logger):
         """Write unique ID of url_data."""
         self.writeln()
         self.write(self.part('id') + self.spaces('id'))
-        self.writeln(u"%d" % self.number, color=self.colorinfo)
+        self.writeln(u"%d" % self.stats.number, color=self.colorinfo)
 
     def write_url (self, url_data):
         """Write url_data.base_url."""
@@ -236,32 +236,53 @@ class TextLogger (Logger):
             self.write(u": " + url_data.result, color=color)
         self.writeln()
 
+    def write_outro (self):
+        self.writeln()
+        self.write(_("That's it.") + " ")
+        self.write(_n("%d link checked.", "%d links checked.",
+                      self.stats.number) % self.stats.number)
+        self.write(u" ")
+        self.write(_n("%d warning found", "%d warnings found",
+             self.stats.warnings_printed) % self.stats.warnings_printed)
+        if self.stats.warnings != self.stats.warnings_printed:
+            self.write(_(" (%d ignored or duplicates not printed)") %
+                (self.stats.warnings - self.stats.warnings_printed))
+        self.write(u". ")
+        self.write(_n("%d error found", "%d errors found",
+             self.stats.errors_printed) % self.stats.errors_printed)
+        if self.stats.errors != self.stats.errors_printed:
+            self.write(_(" (%d duplicates not printed)") %
+                (self.stats.errors - self.stats.errors_printed))
+        self.writeln(u".")
+        self.stoptime = time.time()
+        duration = self.stoptime - self.starttime
+        self.writeln(_("Stopped checking at %(time)s (%(duration)s)") %
+             {"time": strformat.strtime(self.stoptime),
+              "duration": strformat.strduration_long(duration)})
+
+    def write_stats (self):
+        self.writeln()
+        self.writeln(_("Statistics:"))
+        if len(self.stats.domains) > 1:
+            self.writeln(_("Number of domains: %d") % len(self.stats.domains))
+        if self.stats.number > 0:
+            self.writeln(_(
+              "Content types: %(image)d image, %(text)d text, %(video)d video, "
+              "%(audio)d audio, %(application)d application, %(mail)d mail"
+              " and %(other)d other.") % self.stats.link_types)
+            self.writeln(_("URL lengths: min=%(min)d, max=%(max)d, avg=%(avg)d.") %
+                         dict(min=self.stats.min_url_length,
+                         max=self.stats.max_url_length,
+                         avg=self.stats.avg_url_length))
+        else:
+            self.writeln(_("No statistics available since zero URLs were checked."))
+
     def end_output (self):
         """
         Write end of output info, and flush all output buffers.
         """
+        if self.has_part('stats'):
+            self.write_stats()
         if self.has_part('outro'):
-            self.writeln()
-            self.write(_("That's it.") + " ")
-            if self.number >= 0:
-                self.write(_n("%d link checked.", "%d links checked.",
-                              self.number) % self.number)
-                self.write(u" ")
-            self.write(_n("%d warning found", "%d warnings found",
-                            self.warnings_printed) % self.warnings_printed)
-            if self.warnings != self.warnings_printed:
-                self.write(_(" (%d ignored or duplicates not printed)") %
-                    (self.warnings - self.warnings_printed))
-            self.write(u". ")
-            self.write(_n("%d error found", "%d errors found",
-                            self.errors_printed) % self.errors_printed)
-            if self.errors != self.errors_printed:
-                self.write(_(" (%d duplicates not printed)") %
-                    (self.errors - self.errors_printed))
-            self.writeln(u".")
-            self.stoptime = time.time()
-            duration = self.stoptime - self.starttime
-            self.writeln(_("Stopped checking at %(time)s (%(duration)s)") %
-                 {"time": strformat.strtime(self.stoptime),
-                  "duration": strformat.strduration_long(duration)})
+            self.write_outro()
         self.close_fileoutput()
