@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2006 Bastian Kleineidam
+# Copyright (C) 2000-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,18 +11,15 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 """
 Base class for XML loggers.
 """
 
-import time
 import xml.sax.saxutils
-
-import linkcheck.logger
-import linkcheck.configuration
+from . import Logger
 
 
 xmlattr_entities = {
@@ -47,11 +44,8 @@ def xmlquoteattr (s):
     return xml.sax.saxutils.escape(s, xmlattr_entities)
 
 
-class XMLLogger (linkcheck.logger.Logger):
-    """
-    XML output mirroring the GML structure. Easy to parse with any XML
-    tool.
-    """
+class XMLLogger (Logger):
+    """XML output; easy to parse with any XML tool."""
 
     def __init__ (self, **args):
         """
@@ -70,23 +64,14 @@ class XMLLogger (linkcheck.logger.Logger):
         self.write(s, **args)
         self.writeln(u" -->")
 
-    def xml_start_output (self, version="1.0", encoding="utf8"):
+    def xml_start_output (self):
         """
         Write start of checking info as xml comment.
         """
-        self.output_encoding = encoding
-        self.starttime = time.time()
-        self.writeln(u'<?xml version="%s" encoding="%s"?>' %
-                     (xmlquoteattr(version), xmlquoteattr(encoding)))
+        self.writeln(u'<?xml version="1.0" encoding="%s"?>' %
+             xmlquoteattr(self.get_charset_encoding()))
         if self.has_part("intro"):
-            self.comment(_("created by %(app)s at %(time)s") %
-                        {"app": linkcheck.configuration.AppName,
-                         "time": linkcheck.strformat.strtime(self.starttime)})
-            self.comment(_("Get the newest version at %(url)s") %
-                         {'url': linkcheck.configuration.Url})
-            self.comment(_("Write comments and bugs to %(email)s") %
-                         {'email': linkcheck.configuration.Email})
-            self.check_date()
+            self.write_intro()
             self.writeln()
 
     def xml_end_output (self):
@@ -94,11 +79,7 @@ class XMLLogger (linkcheck.logger.Logger):
         Write end of checking info as xml comment.
         """
         if self.has_part("outro"):
-            self.stoptime = time.time()
-            duration = self.stoptime - self.starttime
-            self.comment(_("Stopped checking at %(time)s (%(duration)s)") %
-                 {"time": linkcheck.strformat.strtime(self.stoptime),
-                  "duration": linkcheck.strformat.strduration_long(duration)})
+            self.write_outro()
 
     def xml_starttag (self, name, attrs=None):
         """
@@ -107,10 +88,10 @@ class XMLLogger (linkcheck.logger.Logger):
         self.write(self.indent*self.level)
         self.write(u"<%s" % xmlquote(name))
         if attrs:
-            for name, value in attrs.iteritems():
+            for name, value in attrs.items():
                 args = (xmlquote(name), xmlquoteattr(value))
                 self.write(u' %s="%s"' % args)
-        self.writeln(u">");
+        self.writeln(u">")
         self.level += 1
 
     def xml_endtag (self, name):
@@ -129,7 +110,7 @@ class XMLLogger (linkcheck.logger.Logger):
         self.write(self.indent*self.level)
         self.write(u"<%s" % xmlquote(name))
         if attrs:
-            for aname, avalue in attrs.iteritems():
+            for aname, avalue in attrs.items():
                 args = (xmlquote(aname), xmlquoteattr(avalue))
                 self.write(u' %s="%s"' % args)
         self.writeln(u">%s</%s>" % (xmlquote(content), xmlquote(name)))
