@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2009 Bastian Kleineidam
+# Copyright (C) 2000-2010 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,9 +22,22 @@ import re
 import os
 import urlparse
 import urllib
+import urllib2
+import socket
+from . import httplib2 as httplib
+from . import log, LOG_CHECK
 
 urlparse.uses_netloc.extend(('ldap', 'irc'))
 
+# The character set to encode non-ASCII characters in a URL. See also
+# http://tools.ietf.org/html/rfc2396#section-2.1
+# Note that the encoding is not really specified, but most browsers
+# encode in UTF-8 when no encoding is specified by the HTTP headers,
+# else they use the page encoding for followed link. See als
+# http://code.google.com/p/browsersec/wiki/Part1#Unicode_in_URLs
+url_encoding = "utf-8"
+
+
 # constants defining url part indexes
 SCHEME = 0
 HOSTNAME = DOMAIN = 1
@@ -32,9 +45,9 @@ PORT = 2
 DOCUMENT = 3
 
 default_ports = {
-    'http' : 80,
-    'https' : 443,
-    'nntps' : 563,
+    'http': 80,
+    'https': 443,
+    'nntps': 563,
 }
 
 # adapted from David Wheelers "Secure Programming for Linux and Unix HOWTO"
@@ -215,6 +228,7 @@ def url_fix_host (urlparts):
         urlparts[1] = userpass+host
     return is_idn
 
+
 def url_fix_common_typos (url):
     """Fix common typos in given URL like forgotten colon."""
     if url.startswith("http//"):
@@ -230,21 +244,22 @@ def url_fix_mailto_urlsplit (urlparts):
         urlparts[2], urlparts[3] = urlparts[2].split('?', 1)
 
 
-def url_parse_query (query):
+def url_parse_query (query, encoding=None):
     """Parse and re-join the given CGI query."""
     if isinstance(query, unicode):
-        query = query.encode('iso8859-1', 'ignore')
+        if encoding is None:
+            encoding = url_encoding
+        query = query.encode(encoding, 'ignore')
     # if ? is in the query, split it off, seen at msdn.microsoft.com
-    if '?' in query:
-        query, append = query.split('?', 1)
-        append = '?'+url_parse_query(append)
-    else:
-        append = ""
+    append = ""
+    while '?' in query:
+        query, rest = query.rsplit('?', 1)
+        append = '?'+url_parse_query(rest)+append
     l = []
     for k, v, sep in parse_qsl(query, True):
         k = url_quote_part(k, '/-:,;')
         if v:
-            v = url_quote_part(v, '-:,;')
+            v = url_quote_part(v, '/-:,;')
             l.append("%s=%s%s" % (k, v, sep))
         elif v is None:
             l.append("%s%s" % (k, sep))
@@ -254,13 +269,23 @@ def url_parse_query (query):
     return ''.join(l) + append
 
 
-def url_norm (url):
+def url_norm (url, encoding=None):
     """Normalize the given URL which must be quoted. Supports unicode
     hostnames (IDNA encoding) according to RFC 3490.
 
     @return: (normed url, idna flag)
     @rtype: tuple of length two
     """
+    if isinstance(url, unicode):
+        # try to decode the URL to ascii since urllib.unquote()
+        # handles non-unicode strings differently
+        try:
+            url = url.encode('ascii')
+        except UnicodeEncodeError:
+            pass
+        encode_unicode = True
+    else:
+        encode_unicode = False
     urlparts = list(urlparse.urlsplit(url))
     # scheme
     urlparts[0] = urllib.unquote(urlparts[0]).lower()
@@ -270,7 +295,7 @@ def url_norm (url):
     # host (with path or query side effects)
     is_idn = url_fix_host(urlparts)
     # query
-    urlparts[3] = url_parse_query(urlparts[3])
+    urlparts[3] = url_parse_query(urlparts[3], encoding=encoding)
     is_hierarchical = urlparts[0] not in urlparse.non_hierarchical
     if is_hierarchical:
         # URL has a hierarchical path we should norm
@@ -286,15 +311,15 @@ def url_norm (url):
     # anchor
     urlparts[4] = urllib.unquote(urlparts[4])
     # quote parts again
-    urlparts[0] = url_quote_part(urlparts[0]) # scheme
-    urlparts[1] = url_quote_part(urlparts[1], '@:') # host
-    urlparts[2] = url_quote_part(urlparts[2], _nopathquote_chars) # path
-    urlparts[4] = url_quote_part(urlparts[4]) # anchor
+    urlparts[0] = url_quote_part(urlparts[0], encoding=encoding) # scheme
+    urlparts[1] = url_quote_part(urlparts[1], safechars='@:', encoding=encoding) # host
+    urlparts[2] = url_quote_part(urlparts[2], safechars=_nopathquote_chars, encoding=encoding) # path
+    urlparts[4] = url_quote_part(urlparts[4], encoding=encoding) # anchor
     res = urlparse.urlunsplit(urlparts)
     if url.endswith('#') and not urlparts[4]:
         # re-append trailing empty fragment
         res += '#'
-    if isinstance(url, unicode):
+    if encode_unicode:
         res = unicode(res)
     return (res, is_idn)
 
@@ -351,9 +376,9 @@ def url_quote (url):
     urlparts[3] = url_quote_part(urlparts[3], '&=,') # query
     l = []
     for k, v, sep in parse_qsl(urlparts[3], True): # query
-        k = url_quote_part(k, '/-:,')
+        k = url_quote_part(k, '/-:,;')
         if v:
-            v = url_quote_part(v, '/-:,')
+            v = url_quote_part(v, '/-:,;')
             l.append("%s=%s%s" % (k, v, sep))
         else:
             l.append("%s%s" % (k, sep))
@@ -362,12 +387,13 @@ def url_quote (url):
     return urlparse.urlunsplit(urlparts)
 
 
-def url_quote_part (s, safechars='/'):
+def url_quote_part (s, safechars='/', encoding=None):
     """Wrap urllib.quote() to support unicode strings. A unicode string
-    is first converted to ISO-8859-1, invalid characters are ignored.
-    After that urllib.quote() is called."""
+    is first converted to UTF-8. After that urllib.quote() is called."""
     if isinstance(s, unicode):
-        s = s.encode("iso-8859-1", "ignore")
+        if encoding is None:
+            encoding = url_encoding
+        s = s.encode(encoding, 'ignore')
     return urllib.quote(s, safechars)
 
 def document_quote (document):
@@ -453,3 +479,90 @@ def splitport (host, port=80):
         if iport:
             port = iport
     return host, port
+
+
+class PasswordManager (object):
+    """Simple password manager storing username and password. Suitable
+    for use as an AuthHandler instance in urllib2."""
+
+    def __init__ (self, user, password):
+        """Store given username and password."""
+        self.user = user
+        self.password = password
+
+    def add_password (self, realm, uri, user, passwd):
+        """Does nothing since username and password are already stored.
+
+        @return: None
+        """
+        pass
+
+    def find_user_password (self, realm, authuri):
+        """Get stored username and password.
+
+        @return: A tuple (user, password)
+        @rtype: tuple
+        """
+        return self.user, self.password
+
+
+def get_opener (user=None, password=None, proxy=None, debuglevel=0):
+    """Construct an URL opener object. It considers the given credentials
+    and proxy.
+
+    @return: URL opener
+    @rtype: urllib2.OpenerDirector
+    """
+    from . import httputil
+    pwd_manager = PasswordManager(user, password)
+    handlers = [
+        urllib2.UnknownHandler,
+        httputil.HttpWithGzipHandler(debuglevel=debuglevel),
+        urllib2.HTTPBasicAuthHandler(pwd_manager),
+        urllib2.HTTPDigestAuthHandler(pwd_manager),
+    ]
+    if proxy:
+        handlers.insert(0,
+          urllib2.ProxyHandler({"http": proxy, "https": proxy}))
+        handlers.extend([
+            urllib2.ProxyBasicAuthHandler(pwd_manager),
+            urllib2.ProxyDigestAuthHandler(pwd_manager),
+        ])
+    if hasattr(httplib, 'HTTPS'):
+        handlers.append(httputil.HttpsWithGzipHandler(debuglevel=debuglevel))
+    return urllib2.build_opener(*handlers)
+
+
+def get_content (url, user=None, password=None, proxy=None, data=None):
+    """Get URL content and info.
+
+    @return: (url info, content), or (None, None) on error.
+    @rtype: tuple (string, string)
+    """
+    from . import configuration
+    if log.is_debug(LOG_CHECK):
+        debuglevel = 1
+    else:
+        debuglevel = 0
+    headers = {
+        'User-Agent': configuration.UserAgent,
+        # makes problems with some sites
+        #'Accept-Encoding': 'gzip;q=1.0, deflate;q=0.9, identity;q=0.5',
+    }
+    req = urllib2.Request(url, data, headers)
+    try:
+        f = get_opener(user=user, password=password, proxy=proxy,
+                       debuglevel=debuglevel)
+        res = None
+        try:
+            res = f.open(req)
+            return (res.info(), res.read())
+        finally:
+            if res is not None:
+                res.close()
+    except (urllib2.HTTPError, socket.timeout, urllib2.URLError,
+            socket.gaierror, socket.error, IOError, httplib.HTTPException,
+            ValueError), msg:
+        log.warn(LOG_CHECK, ("Could not get content of URL %(url)s: %(msg)s.") \
+          % {"url": url, "msg": str(msg)})
+    return (None, None)
